# -*- coding: utf-8 -*-
from odoo import http, tools, _
from odoo.addons.portal.controllers.portal import CustomerPortal
from odoo.addons.website_sale.controllers.main import WebsiteSale
from odoo.http import request, route
from odoo.exceptions import ValidationError
from odoo.tools.mimetypes import guess_mimetype

from ..models import consts

import base64
import io
import logging
import random
import werkzeug

from datetime import datetime, timedelta
from markupsafe import Markup
from PIL import Image

_logger = logging.getLogger(__name__)


class OdooPortal(CustomerPortal):

    @http.route(
        '/my/do_certification_as_employee/<int:channel_id>/<int:partner_id>',
        type='http', auth="user", methods=['POST'], website=True
    )
    def do_certification_as_employee(self, channel_id, partner_id, **kw):
        assert request.env.user.has_group('base.group_user')  # is employee

        TAG_CERTIF = request.env.ref('openerp_portal.slide_channel_tag_certification', False)
        certif_channels = request.env['product.product'].search([
            ('name', 'ilike', 'certif'),
            ('sale_ok', '=', True)
        ]).channel_ids.filtered(lambda chan: TAG_CERTIF in chan.tag_ids and chan.enroll == "payment")
        chan = request.env['slide.channel'].sudo().browse(channel_id)
        assert chan in certif_channels, "You cannot add your user in this channel."

        if request.env.user.partner_id.id == partner_id:  # avoid cheating and keep trace in log
            chan._action_add_members(request.env.user.partner_id)
            log = Markup("User <b>%s</b> has been enrolled in the certification <b>%s</b>" % (request.env.user.name, chan.name))
        else:
            log = Markup(
                "!!! User <b>%s</b> try to enroll the partner %s in the certification <b>%s</b>",
                (request.env.user.name, partner_id, chan.name)
            )
            _logger.error(log)
        request.env.user.partner_id.message_post(body=log, message_type='notification')
        return request.redirect(chan.website_url)

    @http.route('/my/do_certification/<so_line>/<token>', type='http', auth="user", website=True)
    def do_certification(self, so_line, token, **kw):
        TAG_CERTIF = request.env.ref('openerp_portal.slide_channel_tag_certification', False)

        error = False
        sol = request.env['sale.order.line'].sudo().search([('id', '=', so_line)], limit=1)
        channel = sol.certif_channel_id

        # Product need to be saleable, contains 'certif' in name and that linked channel has tag 'Certification'
        certif_channels = request.env['product.product'].search([
            ('name', 'ilike', 'certif'),
            ('sale_ok', '=', True)
        ]).channel_ids.filtered(lambda chan: TAG_CERTIF in chan.tag_ids and chan.enroll == "payment")

        if kw.get('channel_id'):
            forced_channel = kw['channel_id'] and channel.browse(int(kw['channel_id'])).exists()
            if forced_channel and forced_channel in certif_channels:
                channel = forced_channel

        if not sol:
            raise werkzeug.exceptions.NotFound()

        if sol.invoice_status == "to invoice":
            error = _("""Your certification order is not yet validated by Odoo.<br/>Be sure that your invoice is marked as paid.""")

        if any(invoice.state in ('draft', 'open') for invoice in sol.order_id.invoice_ids):
            error = _("""You have an unpaid invoice for this order.""")

        if not sol._check_certif_link(token):
            raise werkzeug.exceptions.Forbidden()

        if channel.with_user(request.env.user).is_member:
            error = _("""You are already enrolled for the certification
                         <br/><br/><a class='btn btn-lg btn-primary' href='%s'>Go to certification</a>""", channel.website_url)

        if sol.certif_available <= 0:
            error = _('You have no more certification available.')

        if not error and request.httprequest.method == "POST" and kw.get('Goooo'):
            subscr = channel._action_add_members(request.env.user.partner_id)
            if subscr:
                log = "1 certif used by partner: '%s' (user: %d) for channel: %s [sol:%d] (SlideChannelPartner: %s)" % (
                    request.env.user.partner_id.name, request.env.user.id, channel.name, sol.id, subscr.id
                )
                sol.order_id.message_post(body=log, message_type='notification')
                _logger.info(log)
                sol.write({'certif_available': sol.certif_available - 1})
                return request.redirect(channel.website_url)
            else:
                _logger.warning('UnWanted error in /my/do_certification/%s/%s for user %s', so_line, token, request.env.user)
                error = "Oups, something goes wrong<br/>Please contact the support with your current login, and the current link."

        return request.render('openerp_portal.confirm_certif', {
            'sol': sol,
            'channel': channel,
            'certif_channels': certif_channels,
            'error': Markup(error) if error else False,
        })


class PartnerDashboard(http.Controller):
    def _get_geocountry(self):
        country_code = request.geoip.country_code
        country = country_code and request.env['res.country'].search([('code', '=', country_code)], limit=1)
        return country or request.env.ref('base.be')

    def _get_random_partner(self, country_id):
        CrmTeamMember = request.env['crm.team.member'].sudo()

        team_id = consts.BE_SALES
        country_group_ids = country_id.country_group_ids

        for group in country_group_ids:
            if group.id in consts.TEAM_BY_COUNTRY_GROUP:
                team_id = consts.TEAM_BY_COUNTRY_GROUP[group.id]
                break

        running_saleman = CrmTeamMember.search([('crm_team_id', '=', team_id)])
        if running_saleman:
            contact = random.choice(running_saleman).user_id.partner_id.id
        else:
            contact = request.env['res.partner'].browse(consts.DEFAULT_REPR_PARTNER).id
        return contact

    def _get_success_story_mail(self, country_id):
        ICP = request.env['ir.config_parameter'].sudo()
        CountryGroup = request.env['res.country.group']
        if country_id in CountryGroup.browse(consts.EMEA).exists().country_ids:
            mail = ICP.get_param('success_story.email.emea', "sng@odoo.com")
        elif country_id in CountryGroup.browse(consts.APAC).exists().country_ids:
            mail = ICP.get_param('success_story.email.apac', "fyu@odoo.com")
        else:
            mail = ICP.get_param('success_story.email.other', "tve@odoo.com")
        return mail

    def _values(self, access_token):
        Subscription = request.env['sale.order'].sudo()
        Partner = request.env['res.partner'].sudo()
        Visitor = request.env['website.visitor'].sudo()
        visitor_sudo = Visitor._get_visitor_from_request(force_create=True)
        lead = visitor_sudo.lead_ids[:1]
        subscription = False
        saleman = False
        partner = False
        contact = False
        lead_pic = False

        partnership_tag = request.env.ref('openerp_enterprise.odoo_partnership_tag')
        if access_token:
            subscription = Subscription.search([
                ('is_subscription', '=', True),
                ('access_token', '=', access_token),
                ('sale_order_template_id.subscription_tag_ids', 'in', [partnership_tag.id])],
            limit=1, order='create_date desc')

        if not subscription and not request.website.is_public_user():
            subscription = Subscription.search([
                ('is_subscription', '=', True),
                ('partner_id', '=', request.env.user.partner_id.commercial_partner_id.id),
                ('sale_order_template_id.subscription_tag_ids', 'in', [partnership_tag.id]),
                ('subscription_state', '=', '3_progress')
            ], limit=1, order='create_date desc')

        if subscription:
            partner = subscription.partner_id
            saleman = partner.commercial_partner_id.user_id.partner_id.id
        else:
            if request.website.is_public_user():
                if lead:
                    lead_pic = lead.env['ir.attachment'].search([
                        ('name', '=', 'partner_dashboard_picture'),
                        ('res_id', '=', lead.id),
                        ('res_model', '=', 'crm.lead')
                    ])['datas']
                    saleman = lead.user_id.id or (lead.partner_id and lead.partner_id.user_id.id)

                    contact = Partner.new({
                        'name': lead.contact_name,
                        'street': lead.street,
                        'state_id': lead.state_id,
                        'city': lead.city,
                        'zip': lead.zip,
                        'country_id': lead.country_id,
                        'email': lead.email_from,
                        'phone': lead.phone,
                    })
            else:
                partner = request.env.user.partner_id.commercial_partner_id
                saleman = partner.user_id.partner_id.id

        country = (partner and partner.country_id) or (lead and lead.country_id) or self._get_geocountry()

        if not saleman:
            saleman = request.session.get('saleman_id') or self._get_random_partner(country)
            request.session['saleman_id'] = saleman

        values = {}
        values.update(country._get_country_cached_stat_dashboard(generate_if_no_data=request.params.get('debug')))
        values.update(country._get_events_dashboard())

        values.update({
            'saleman': Partner.browse(saleman),
            'access_token': access_token,
            'partner': partner,
            'contact': contact or partner,
            'country_id': country,
            'source_image': (partner and partner.image_128) or lead_pic,
            'mail': request.env['discuss.channel'].browse(consts.CHANNEL_ALL_EMPLOYEES),
            'lead': lead,
            'success_story_mail': self._get_success_story_mail(country)
        })

        if partner:
            is_employee = request.env.user.partner_id.commercial_partner_id == partner.commercial_partner_id
            # only compute subs, purchases and leads data if there is an ongoing Odoo partnership
            values.update(partner._get_subscriptions_dashboard())
            values.update(partner._get_purchases_dashboard())
            values.update(partner._get_leads_dashboard())
            values.update(partner._get_grade_dashboard())
            values['my_sub'] = subscription
            values['is_employee'] = is_employee
            values['is_accounting_firm'] = partner.portal_accounting_firm_level
            request.session['partner_id'] = partner.id

            values['new_user_details'] = partner.sudo()._get_new_enterprise_user_details()
            values['retention_rate_details'] = partner.sudo()._get_retention_details()

        # compute eLearning progress
        lower_bound = request.env.user.rank_id.karma_min
        upper_bound = request.env.user.next_rank_id.karma_min
        try:
            values['elearning_progress'] = 100 * ((request.env.user.karma - lower_bound) / (upper_bound - lower_bound))
        except ZeroDivisionError:  # should not happen unless karma_min are misconfigured
            values['elearning_progress'] = 0
        values['elearning_rank'] = request.env.user.rank_id
        return values

    @http.route(['/my/partner', '/my/partner/<access_token>'], type='http', auth="public", website=True, sitemap=True)
    def dashboard_token(self, access_token=None, **kw):
        values = self._values(access_token)
        return request.render('openerp_portal.dashboard', values)

    @http.route(['/my/partner/saleman', '/my/partner/saleman/<access_token>'],
                type='http', auth="public", website=True, sitemap=False)
    def dashboard_saleman(self, access_token=None, **post):
        values = self._values(access_token)
        img_stream = http.Stream.from_binary_field(
            values['saleman'].sudo(), 'image_128',  # force partner img even if not published
        )
        img_stream.mimetype = guess_mimetype(img_stream.data, default='image/png')
        return img_stream.get_response()

    @http.route(['/my/partner/profile'], type='http', auth='public', website=True, sitemap=True)
    def dasboard_profile(self, redirect=None, **kw):
        Lead = request.env['crm.lead'].sudo()
        visitor = request.env['website.visitor'].sudo()._get_visitor_from_request(force_create=True)
        lead = visitor.lead_ids[0] if visitor.lead_ids else False
        MANDATORY_FIELDS = ["name", "phone", "email", "street", "city", "country_id"]
        OPTIONAL_FIELDS = ["zipcode", "state_id", "vat", "company_name", "website_short_description", "website_description"]

        # filter allowed values that we provide as context of the rendering
        data = {k: kw[k] for k in kw if k in (MANDATORY_FIELDS + OPTIONAL_FIELDS + ['image'])}

        error = {}
        error_message = []
        country = False

        if request.website.is_public_user():
            partner = False
            img = lead and lead.env['ir.attachment'].search([
                ('name', '=', 'partner_dashboard_picture'), ('res_id', '=', lead.id)
            ])['datas']
            if not lead:
                country = self._get_geocountry()
        else:
            partner = request.env.user.partner_id.commercial_partner_id
            img = partner.image_128
            lead = False

        if request.httprequest.method == 'POST':

            # Validation required
            for field_name in MANDATORY_FIELDS:
                if not data.get(field_name):
                    error[field_name] = 'missing'

            # email validation
            if data.get('email') and not tools.single_email_re.match(data.get('email')):
                error["email"] = 'error'
                error_message.append(_('Invalid Email! Please enter a valid email address.'))

            # vat validation
            Partner = request.env["res.partner"]
            if data.get("vat") and hasattr(Partner, "check_vat"):
                if data.get("country_id"):
                    data["vat"] = request.env["res.partner"].fix_eu_vat_number(int(data.get("country_id")), data.get("vat"))
                partner_dummy = Partner.new({
                    'vat': data['vat'],
                    'country_id': (int(data['country_id']) if data.get('country_id') else False),
                })
                try:
                    partner_dummy.check_vat()
                except ValidationError:
                    error["vat"] = 'error'

            # File Validation
            if data['image']:
                new_img = base64.encodebytes(request.httprequest.files['image'].stream.read())
                try:
                    test_img = Image.open(data['image'])
                    test_img.verify()
                except IOError:
                    error['image'] = 'error'
                    error_message.append(_('Invalid type of file! Please give a valid image file.'))

            # error message for empty required fields
            if [err for err in error.values() if err == 'missing']:
                error_message.append(_('Some required fields are empty.'))

            if not error:
                resp = request.redirect('/my/partner')
                values = {key: data[key] for key in MANDATORY_FIELDS}
                values.update({key: data[key] or False for key in OPTIONAL_FIELDS if key in data})
                if data['image']:
                    values['image_1920'] = new_img

                values['zip'] = values.pop('zipcode', '')
                values['country_id'] = int(values['country_id'])
                values['state_id'] = values.get('state_id') and int(values['state_id'])
                if not partner:
                    lead_data = {
                        'partner_name': values['company_name'],
                        'street': values['street'],
                        'city': values['city'],
                        'zip': values['zip'],
                        'country_id': values['country_id'],
                        'email_from': values['email'],
                        'phone': values['phone'],
                        'contact_name': values['name'],
                        'state_id': values['state_id'],
                        'function': values['vat'],  # save vat into function field to allow to keep it for checkout
                    }
                    if lead:
                        lead_data['user_id'] = lead.user_id
                        lead.write(lead_data)
                    else:
                        new_partnership_tag = request.env.ref('openerp_portal.partnership_dashboard', False)
                        lead_data.update({
                            'name': _("New Partnership for %s", values['name']),
                            'user_id': False,
                            'team_id': False,
                            'tag_ids': [(6, 0, [new_partnership_tag.id, Lead.get_tag_id('plan_to_sell')])]
                        })
                        lead = Lead.create(lead_data)
                        visitor.write({'lead_ids': [(4, lead.id)]})
                        expiration_date = datetime.now() + timedelta(days=365)
                        resp.set_cookie('visitor_uuid', visitor.access_token, expires=expiration_date)

                    if data['image']:
                        attach_id = lead.env['ir.attachment'].search([('res_id', '=', lead.id), ('name', '=', 'partner_dashboard_picture')])
                        if attach_id:
                            attach_id.datas = values['image_1920']
                        else:
                            request.env['ir.attachment'].sudo().create({
                                'datas': values['image_1920'],
                                'name': 'partner_dashboard_picture',
                                'res_model': 'crm.lead',
                                'res_id': lead.id,
                            })
                else:
                    if not partner.can_edit_vat():
                        values.pop('vat', False)
                        values.pop('name', False)
                        values.pop('company_name', False)
                    partner = partner.write(values)
                return resp

        if partner and partner.commercial_partner_id == partner:
            data['company_name'] = partner.name
            data['vat'] = partner.vat

        data.update({
            'partner': partner or request.env["res.partner"],
            'ACCOUNTING_FIRM_NOPUBLISH': consts.ACCOUNTING_FIRM_NOPUBLISH,
            'source_email': lead and lead.email_from or partner and partner.email,
            'source_name': lead and lead.contact_name or partner and partner.name,
            'source_image': img,
            'source_vat': lead and lead.function or partner and partner.vat,
            'source_company_name': lead and lead.partner_name or partner and partner.company_name,
            'source': partner or lead or request.env["res.partner"],
            'countries': request.env['res.country'].search([]),
            'has_check_vat': hasattr(request.env['res.partner'], 'check_vat'),
            'error': error,
            'error_message': error_message,
            'country_id': country,
        })

        return request.render("openerp_portal.partner_form", data)

    @http.route('/become-a-partner/pricing', type='http', auth="public", website=True, sitemap=True)
    def pricing(self, **kw):
        Product = request.env['product.template'].sudo()

        learning_price = Product.browse(consts.PRODUCT_PARTNERSHIP_LEARNING)
        official_price = Product.browse(consts.PRODUCT_PARTNERSHIP_OFFICIAL)
        discount_subscription_official = Product.browse(consts.PRODUCT_DISCOUNT_SUBSCRIPTION_OFFICIAL)
        discount_subscription_learning = Product.browse(consts.PRODUCT_DISCOUNT_SUBSCRIPTION_LEARNING)
        commission_odoosh = request.env['commission.plan'].sudo().search([], order="id desc", limit=1).commission_rule_ids.filtered(
            lambda x: x.category_id == request.env.ref('openerp_enterprise.product_category_odoo_sh')
                and x.template_id == request.env.ref('openerp_enterprise.custom_template_odoo_monthly')
                and x.pricelist_id == request.website.pricelist_id
        )

        return request.render('openerp_portal.plans_prices', {
            'discount_subscription_official': discount_subscription_official,
            'discount_subscription_learning': discount_subscription_learning,
            'learning_price': learning_price,
            'official_price': official_price,
            'commission_odoosh': commission_odoosh.max_commission,
            'buy_now_disabled': request.env.user.partner_id.grade_id.id == consts.LEARNING_GRADE,
        })

    @http.route('/my/partner/toggle_publish', type='json', auth="user", website=True)
    def publish_partner(self):
        com_partner = request.env.user.partner_id.commercial_partner_id
        is_partner_grade_published = com_partner.grade_id.id in consts.GRADE_ALLOW_PUBLISH
        is_account_level_published = com_partner.portal_accounting_firm_level not in consts.ACCOUNTING_FIRM_NOPUBLISH
        if com_partner and (is_partner_grade_published or is_account_level_published):
            com_partner.sudo().website_published = not com_partner.website_published
            return com_partner.website_published
        return False

    @http.route('/my/partner/stat_by_country/<model("res.country"):country>', type='json', auth="public", website=True)
    def dashboard_stat_by_country(self, country):
        return country._get_country_cached_stat_dashboard()

    @http.route('/become-a-partner', type='http', auth='public', website=True, sitemap=True)
    def home_become_a_partner(self, **kw):
        values = {}
        values['country_id'] = self._get_geocountry()
        monthly_lead = values['country_id']._get_country_cached_stat_dashboard()['country_leads'] // 12
        values['leads'] = monthly_lead
        if values['leads'] < consts.MIN_LEADS_BY_COUNTRY:
            monthly_lead = values['country_id']._get_country_cached_stat_dashboard(country_id=0)['country_leads'] / 12
            values['leads'] = int(monthly_lead // 100) * 100
            values['country_id'] = 'Odoo'

        commission_odoosh = request.env['commission.plan'].sudo().search([], order="id desc", limit=1).commission_rule_ids.filtered(
            lambda x: x.category_id == request.env.ref('openerp_enterprise.product_category_odoo_sh')
                and x.template_id == request.env.ref('openerp_enterprise.custom_template_odoo_monthly')
                and x.pricelist_id == request.website.pricelist_id
        )

        Product = request.env['product.template'].sudo()
        values['discount_subscription_learning'] = Product.browse(consts.PRODUCT_DISCOUNT_SUBSCRIPTION_LEARNING)
        values['discount_subscription_official'] = Product.browse(consts.PRODUCT_DISCOUNT_SUBSCRIPTION_OFFICIAL)
        values['commission_odoosh'] = commission_odoosh.max_commission

        return request.render("openerp_portal.become_a_partner", values)

    @route('/my/partner/github/add', type='http', auth='public', methods=['POST'])
    def add_github_user(self, github_username, sub_uuid, **kwargs):
        GithubUser = request.env['openerp.enterprise.github.user'].sudo()
        Subscription = request.env['sale.order'].sudo()
        partnership_tag = request.env.ref('openerp_enterprise.odoo_partnership_tag')
        subscription = Subscription.search([
                ('is_subscription', '=', True),
                ('access_token', '=', sub_uuid),
                ('sale_order_template_id.subscription_tag_ids', 'in', [partnership_tag.id])],
            limit=1, order='create_date desc')

        if not subscription:
            raise werkzeug.exceptions.Forbidden()
        if len(subscription.github_user_ids) >= subscription.github_limit:
            raise werkzeug.exceptions.Forbidden()
        existing_github_user = GithubUser.search([('name', '=', github_username)])
        if existing_github_user.status == 'none':
            existing_github_user.unlink()
        elif existing_github_user:
            return request.redirect('/my/partner/%s?gh_uniq=%s#develop_and_deploy' % (subscription.access_token, github_username))
        GithubUser.create({'name': github_username, 'subscription_id': subscription.id})
        return request.redirect('/my/partner/%s#develop_and_deploy' % subscription.access_token)


class WebsiteSaleDashboard(WebsiteSale):
    @http.route('/become-a-partner/checkout', auth='public', website=True, sitemap=False)
    def partnership_pricing_checkout(self, **kw):

        visitor = request.env['website.visitor']._get_visitor_from_request()
        lead = None
        if visitor:
            lead = visitor.lead_ids[:1]

        request.website.odoo_get_order(periodicity="yearly_learning", force_create=True)

        if request.website.is_public_user() and lead:
            values = {
                'submitted': 'submitted',
                'street': lead.street,
                'city': lead.city,
                'zip': lead.zip,
                'country_id': lead.country_id.id,
                'email': lead.email_from,
                'phone': lead.phone,
                'name': lead.contact_name,
                'state_id': lead.state_id.id,
                'vat': lead.function,
                'company_name': lead.partner_name,
            }
            return self.address(**values)
        else:
            return request.redirect('/shop/checkout')

    def values_postprocess(self, order, mode, values, errors, error_msg):
        first_view = request.httprequest.method == "GET"
        sold_elearning = consts.PRODUCT_PARTNERSHIP_LEARNING in order.order_line.mapped('product_id.product_tmpl_id').ids
        new_values, errors, error_msg = super(WebsiteSaleDashboard, self).values_postprocess(order, mode, values, errors, error_msg)
        if first_view and not error_msg and mode == ('new', 'billing') and sold_elearning and values.get('company_name') and not values.get('vat'):
            errors['vat'] = '1'
            error_msg.append(_("Do you have a TIN / VAT?"))
        return new_values, errors, error_msg
